"""
Fabric 2 implementations of file routines that
used to come with Fabric 1.
"""

import io
import pathlib
import tempfile


def is_dir(c, candidate):
    cmd = f'test -d "{candidate}"'
    return c.run(cmd, warn=True)


def exists(c, candidate):
    cmd = f'test -f "{candidate}"'
    return c.run(cmd, warn=True)


def upload_template(c, src, dest, *, mode=None, context={}):
    rnd_name = next(tempfile._get_candidate_names())
    tmp_dest = f'/tmp/{rnd_name}'
    template = pathlib.Path(src).read_text()
    content = template % context
    stream = io.StringIO(content)
    c.put(stream, tmp_dest)
    if is_dir(c, dest):
        dest = pathlib.PurePosixPath(dest, pathlib.Path(src).name)
    c.sudo(f'mv "{tmp_dest}" "{dest}"')
    if mode is not None:
        mode_str = oct(mode)[2:]
        c.run(f'chmod {mode_str} {dest}')



import itertools

from jaraco.home import contact

flatten = itertools.chain.from_iterable


def install(c, *sites):
    cmd = [
        'certbot',
        '--agree-tos',
        '--email',
        contact.load().email,
        '--non-interactive',
        '--nginx',
        'certonly',
    ]
    cmd += list(flatten(['--domain', name] for name in sites))
    c.sudo(' '.join(cmd))



from __future__ import annotations

extensions = [
    'sphinx.ext.autodoc',
    'jaraco.packaging.sphinx',
]

master_doc = "index"
html_theme = "furo"

# Link dates and other references in the changelog
extensions += ['rst.linker']
link_files = {
    '../NEWS.rst': dict(
        using=dict(GH='https://github.com'),
        replace=[
            dict(
                pattern=r'(Issue #|\B#)(?P<issue>\d+)',
                url='{package_url}/issues/{issue}',
            ),
            dict(
                pattern=r'(?m:^((?P<scm_version>v?\d+(\.\d+){1,2}))\n[-=]+\n)',
                with_scm='{text}\n{rev[timestamp]:%d %b %Y}\n',
            ),
            dict(
                pattern=r'PEP[- ](?P<pep_number>\d+)',
                url='https://peps.python.org/pep-{pep_number:0>4}/',
            ),
        ],
    )
}

# Be strict about any broken references
nitpicky = True
nitpick_ignore: list[tuple[str, str]] = []

# Include Python intersphinx mapping to prevent failures
# jaraco/skeleton#51
extensions += ['sphinx.ext.intersphinx']
intersphinx_mapping = {
    'python': ('https://docs.python.org/3', None),
}

# Preserve authored syntax for defaults
autodoc_preserve_defaults = True

# Add support for linking usernames, PyPI projects, Wikipedia pages
github_url = 'https://github.com/'
extlinks = {
    'user': (f'{github_url}%s', '@%s'),
    'pypi': ('https://pypi.org/project/%s', '%s'),
    'wiki': ('https://wikipedia.org/wiki/%s', '%s'),
}
extensions += ['sphinx.ext.extlinks']

# local



import contextlib
import getpass
import io
import json
import os
import re

import yaml
from importlib_resources import files

from fabric import task

from . import apt

__all__ = (
    'distro_install',
    'find_current_version',
    'install_systemd',
    'enable_authentication',
    'install_user',
    'bind_all',
)


APT_KEYS = ['EA312927']


@task
def distro_install(c, version="3.2"):
    """
    Install mongodb as an apt package (which also configures it as a
    service).
    """

    if version.startswith('2.'):
        return distro_install_2(version)

    elif version.startswith('3.'):
        return distro_install_3(version)

    raise RuntimeError('Unknown version {}'.format(version))


def append(c, *args, **kwargs):
    # todo: used to be contrib.files.append, now patchwork.files, not ported
    raise NotImplementedError()


def distro_install_2(c, version):
    """
    Install MongoDB version 2.x
    """
    assert version.startswith('2.')

    # MongoDB 2
    content = (
        'deb http://downloads-distro.mongodb.org/repo/ubuntu-upstart dist 10gen',
    )
    org_list = '/etc/apt/sources.list.d/mongodb.list'

    append(c, org_list, content, use_sudo=True)


@task
def find_current_version(c):
    output = c.sudo('apt list -qq mongodb-org')
    return re.search(r'\d+\.\d+\.\d+', output).group(0)


def distro_install_3(c, version):
    """
    Install MongoDB version 3.x
    """
    assert version.startswith('3.')

    lsb_release = apt.lsb_release()
    repo_url = "http://repo.mongodb.org/apt/ubuntu"
    tmpl = "deb {repo_url} {lsb_release}/mongodb-org/{version} multiverse"
    content = tmpl.format(**locals())
    org_list = f'/etc/apt/sources.list.d/mongodb-org-{version}.list'
    append(c, org_list, content, use_sudo=True)

    for key in APT_KEYS:
        c.sudo(
            f'apt-key adv --keyserver hkp://keyserver.ubuntu.com:80 --recv {key}',
            warn=True,
        )

    c.sudo('apt update')
    version = find_current_version()
    apt.install_packages('mongodb-org={version}'.format(**locals()))
    install_systemd()


@task
def enable_authentication(c):
    with yaml_config(c, '/etc/mongod.conf', use_sudo=True) as config:
        security = config.setdefault('security', {})
        security['authorization'] = 'enabled'


@task
def bind_all(c):
    """
    Bind to all interfaces.
    """
    with yaml_config('/etc/mongod.conf', use_sudo=True) as config:
        config.setdefault('net', {}).update(bindIp='0.0.0.0')


@contextlib.contextmanager
def yaml_config(c, path, use_sudo=False):
    cmd = c.sudo if use_sudo else c.run
    doc = yaml.safe_load(cmd('cat {path}'.format(**locals())))
    yield doc
    new_doc = io.StringIO(yaml.dump(doc, default_flow_style=False))
    c.put(new_doc, path, use_sudo=use_sudo)


@task
def install_user(c, username=None):
    default = os.environ['USER']
    username = username or getpass.getuser()
    password = getpass.getpass('password> ')
    new_user = dict(
        user=username,
        pwd=password,
        roles=[dict(role="userAdminAnyDatabase", db="admin")],
    )
    cmd = 'db.createUser({doc})'.format(doc=json.dumps(new_user))
    c.run('mongo admin -eval {cmd!r}'.format(**locals()))


@task
def install_systemd(c):
    """
    On newer versions of Ubuntu, make sure that systemd is configured
    to manage the service.
    https://docs.mongodb.com/v3.2/tutorial/install-mongodb-on-ubuntu/#ubuntu-16-04-only-create-systemd-service-file
    """
    if apt.lsb_version() < '15.04':
        return

    fn = 'mongod.service'
    service_path = files().joinpath(fn)
    with service_path.open(mode='rb') as strm:
        c.put(strm, '/lib/systemd/system/' + fn, use_sudo=True)
    c.sudo('systemctl enable mongod')
    # TODO: does the service start automatically? If not,
    # sudo('systemctl start mongod')



import functools


@functools.cache
def get_shell(c):
    return c.run('echo $SHELL', hide=True).stdout


def escape_param(c, param):
    """
    Some parameters need shell-specific escaping.
    """
    shell = get_shell(c)
    if 'xonsh' in shell:
        return repr(param)
    return f"'{param}'"



import functools


def workaround_2090(func):
    """
    Wrap a task func to work around fabric/fabric#2090.
    """

    @functools.wraps(func)
    def wrapper(c, *args, **kwargs):
        if 'key_filename' in c.connect_kwargs:
            c.connect_kwargs['key_filename'][:] = [
                key
                for key in c.connect_kwargs['key_filename']
                if __import__('os').path.exists(key)
            ]
        return func(c, *args, **kwargs)

    return wrapper
